import base64
import codecs
import os
import re
import zlib

import xbmc
import xbmcvfs

from resources.lib.const import URL, SETTINGS, lang_to_iso_3, LANG
import xmlrpclib

from resources.lib.kodilogging import logger
from resources.lib.storage.settings import settings
from resources.lib.utils.kodiutils import get_string

codePageDict = {
    'ara': 'cp1256',
    'ar': 'cp1256',
    'cs': 'cp1250',
    'ell': 'cp1253',
    'el': 'cp1253',
    'heb': 'cp1255',
    'he': 'cp1255',
    'sk': 'cp1250',
    'tur': 'cp1254',
    'tr': 'cp1254',
    'rus': 'cp1251',
    'ru': 'cp1251'
}

class OpenSubtitles:
    def __init__(self):
        self.server = xmlrpclib.Server(URL.OPEN_SUBTITLES, verbose=0, allow_none=True)
        self._token = settings[SETTINGS.SUBTITLES_PROVIDER_TOKEN]

    @staticmethod
    def get_languages():
        return [lang_to_iso_3[lang] for lang in settings.get_languages(SETTINGS.SUBTITLES_PROVIDER_PREFERRED_LANGUAGE,
                                                                       SETTINGS.SUBTITLES_PROVIDER_FALLBACK_LANGUAGE)]

    def login(self, username=settings[SETTINGS.SUBTITLES_PROVIDER_USERNAME], password=settings[SETTINGS.SUBTITLES_PROVIDER_PASSWORD]):
        response = self.server.LogIn(username, password, 'en', 'XBMC_Subtitles_v1')
        settings[SETTINGS.SUBTITLES_PROVIDER_TOKEN] = response['token']
        logger.debug('Getting token from OpenSubtitles')
        return response

    def search(self, media):
        languages = OpenSubtitles.get_languages()
        info_labels = media.get('info_labels', {})
        if not self._token:
            self.login()

        result = self.server.SearchSubtitles(self._token, [{
            'sublanguageid': ','.join(languages),
            'imdbid': media.get('services', {}).get('imdb')[2:],
            'season': info_labels.get('season'),
            'episode': info_labels.get('episode'),
        }])
        for language in languages:
            for data in result.get('data', []):
                if data.get('SubLanguageID') == language:
                    return data

    def download(self, sub):
        logger.debug('Downloading subtitles %s', sub.get('IDSubtitleFile'))
        content = self.server.DownloadSubtitles(self._token, [sub.get('IDSubtitleFile')])
        content = base64.b64decode(content['data'][0]['data'])
        content = zlib.decompressobj(16 + zlib.MAX_WBITS).decompress(content)
        subtitle = xbmc.translatePath('special://temp/')
        subtitle = os.path.join(subtitle, get_string(LANG.AUTO_SUBTITLES) + '.%s.srt' % sub.get('sublanguageid'))
        codepage = codePageDict.get(sub.get('ISO639'), '')
        if codepage:
            try:
                content_encoded = codecs.decode(content, codepage)
                content = codecs.encode(content_encoded, 'utf-8')
            except:
                pass
            file = xbmcvfs.File(subtitle, 'w')
            file.write(str(content))
            file.close()
            return subtitle


OpenSubtitles = OpenSubtitles()
